@@ -1,7 +1,7 @@
 /*******************************************************************************
 *                         Goggles Music Manager                                *
 ********************************************************************************
-*           Copyright (C) 2007-2010 by Sander Jansen. All Rights Reserved      *
+*           Copyright (C) 2007-2021 by Sander Jansen. All Rights Reserved      *
 *                               ---                                            *
 * This program is free software: you can redistribute it and/or modify         *
 * it under the terms of the GNU General Public License as published by         *
@@ -18,28 +18,10 @@
 ********************************************************************************/
 #include "gmdefs.h"
 #include "gmutils.h"
-
-
-
-#include "ap_buffer.h"
-#include "ap_http.h"
-#include "ap_xml_parser.h"
-
-
+#include "GMApp.h"
+#include "GMTrack.h"
 #include "GMAudioScrobbler.h"
-
-
-
-#ifdef HAVE_GCRYPT
-#include "gcrypt.h"
-#else
-#include "md5.h"
-#endif
-
-
-
-using namespace ap;
-
+#include "GMAudioPlayer.h"
 
 /******************************************************************************
  *
@@ -60,7 +42,7 @@ using namespace ap;
 #define CLIENT_KEY "76525254135cc544d13d381514222c56"
 #define CLIENT_SECRET "09397d5d6a55858a6883735b7cb694f7"
 
-#define SCROBBLER_CACHE_FILE PATHSEPSTRING ".goggles" PATHSEPSTRING "scrobbler.cache"
+#define SCROBBLER_CACHE_FILE PATHSEPSTRING "scrobbler.cache"
 
 #define LASTFM_URL "http://ws.audioscrobbler.com:80/2.0/"
 #define LASTFM_OLD_URL "http://post.audioscrobbler.com:80"
@@ -109,52 +91,6 @@ enum  {
 */
 
 
-/******************************************************************************
- *
- * H E L P E R  F U N C T I O N S
- *
- ******************************************************************************/
-
-FXbool init_gcrypt() {
-#ifdef HAVE_GCRYPT
-  if (!gcry_check_version(GCRYPT_VERSION)) {
-    fxwarning("libgcrypt version mismatch");
-    return false;
-    }
-  gcry_control(GCRYCTL_DISABLE_SECMEM,0);
-  gcry_control(GCRYCTL_INITIALIZATION_FINISHED,0);
-#endif
-  return true;
-  }
-
-
-static void checksum(FXString & io){
-  if (io.empty()) return;
-#ifdef HAVE_GCRYPT
-  FXuchar digest[16];
-  gcry_md_hash_buffer(GCRY_MD_MD5,(void*)digest,(const void*)io.text(),io.length());
-#else
-  md5_state_t pms;
-  md5_byte_t digest[16];
-  md5_init(&pms);
-  md5_append(&pms,(const md5_byte_t*)io.text(),io.length());
-  md5_finish(&pms,digest);
-#endif
-
-  io.length(32);
-#if FOXVERSION < FXVERSION(1,7,0)
-  for (FXint i=0,d=0;i<32;i+=2,d++) {
-    io[i]=FXString::hex[(digest[d]/16)%16];
-    io[i+1]=FXString::hex[digest[d]%16];
-    }
-#else
-  for (FXint i=0,d=0;i<32;i+=2,d++) {
-    io[i]=Ascii::toLower(FXString::value2Digit[(digest[d]/16)%16]);
-    io[i+1]=Ascii::toLower(FXString::value2Digit[digest[d]%16]);
-    }
-#endif
-  }
-
 #define URL_UNSAFE   "#$-_.+!*'><()\\,%\""          // Always Encode
 #define URL_RESERVED ";/?:@=&"              // Only encode if not used as reserved by scheme
 
@@ -193,15 +129,13 @@ void GMAudioScrobblerTrack::load(FXStream & store) {
   }
 
 
-class ServiceResponse : public XMLStream {
+class ServiceResponse : public XmlParser {
 protected:
   FXbool       status;
   FXint        code;
   FXString     message;
   FXString     key;
   FXString     token;
-protected:
-  FXint        elem;
 protected:
   FXint begin(const FXchar *,const FXchar**);
   void data(const FXchar *,FXint len);
@@ -210,8 +144,7 @@ protected:
   void parse_lfm_error_atts(const FXchar **);
 public:
   enum {
-    Elem_None,
-    Elem_LastFM,
+    Elem_LastFM = Elem_Last,
     Elem_LastFM_Error,
     Elem_LastFM_Token,
     Elem_LastFM_Session,
@@ -229,7 +162,7 @@ public:
   };
 
 
-ServiceResponse::ServiceResponse() : elem(Elem_None) {
+ServiceResponse::ServiceResponse() : status(false), code(LASTFM_ERROR_UNKNOWN) {
    }
 
 ServiceResponse::~ServiceResponse() {
@@ -253,43 +186,38 @@ void ServiceResponse::parse_lfm_error_atts(const FXchar ** attributes) {
   for (FXint i=0;attributes[i];i+=2){
     if (compare(attributes[i],"code")==0) {
       cd=attributes[i+1];
-      code=GMIntVal(cd);
+      code=cd.toInt();
       }
     }
   }
 
 
 FXint ServiceResponse::begin(const FXchar * element,const FXchar ** attributes){
-  switch(elem) {
+  switch(node()) {
     case Elem_None:
       {
         if (compare(element,"lfm")==0) {
           parse_lfm_atts(attributes);
-          elem=Elem_LastFM;
-          return 1;
+          return Elem_LastFM;
           }
       } break;
     case Elem_LastFM:
       {
         if (compare(element,"error")==0) {
           parse_lfm_error_atts(attributes);
-          elem=Elem_LastFM_Error;
-          return 1;
+          return Elem_LastFM_Error;
           }
         else if (compare(element,"session")==0) {
-          elem=Elem_LastFM_Session;
-          return 1;
+          return Elem_LastFM_Session;
           }
         else if (compare(element,"token")==0) {
-          elem=Elem_LastFM_Token;
-          return 1;
+          return Elem_LastFM_Token;
           }
       } break;
     case Elem_LastFM_Session:
       {
         if (compare(element,"key")==0) {
-          elem=Elem_LastFM_Session_Key;
-          return 1;
+          return Elem_LastFM_Session_Key;
           }
       } break;
     default: return 0; // skip
@@ -298,38 +226,27 @@ FXint ServiceResponse::begin(const FXchar * element,const FXchar ** attributes){
   }
 
 
-void ServiceResponse::data(const FXchar* data,FXint len){
-  if (elem==Elem_LastFM_Error) {
-    message.assign(data,len);
-    }
-  else if (elem==Elem_LastFM_Session_Key) {
-    key.assign(data,len);
-    }
-  else if (elem==Elem_LastFM_Token) {
-    token.assign(data,len);
+void ServiceResponse::data(const FXchar* d,FXint len){
+  switch(node()) {
+    case Elem_LastFM_Error: message.assign(d,len); break;
+    case Elem_LastFM_Session_Key: key.assign(d,len); break;
+    case Elem_LastFM_Token: token.assign(d,len); break;
     }
   }
 
 void ServiceResponse::end(const FXchar*) {
-  switch(elem){
-    case Elem_LastFM_Session_Key : elem=Elem_LastFM_Session; break;
-    case Elem_LastFM_Token       :
-    case Elem_LastFM_Error       :
-    case Elem_LastFM_Session     : elem=Elem_LastFM; break;
-    case Elem_LastFM             : elem=Elem_None; break;
-    }
   }
 
 
 
-FXbool GMAudioScrobbler::post_request(const FXString & url,const FXString & message,FXString & output) {
-  FXTRACE((60,"post_request %s - %s\n",url.text(),message.text()));
+FXbool GMAudioScrobbler::post_request(const FXString & url,const FXString & msg,FXString & output) {
+  FXTRACE((60,"post_request %s - %s\n",url.text(),msg.text()));
   HttpClient client;
   if (client.basic("POST",url,
-                      "Content-Type: application/x-www-form-urlencoded\r\n"
-                      "Connection: close\r\n",
-                       message)) {
-    output = client.body();
+                   "Content-Type: application/x-www-form-urlencoded\r\n"
+                   "Connection: close\r\n",
+                   msg)) {
+    output = client.textBody();
     return true;
     }
   else {
@@ -343,7 +260,7 @@ FXbool GMAudioScrobbler::get_request(const FXString & url,FXString & output) {
   FXTRACE((60,"get_request\n"));
   HttpClient client;
   if (client.basic("GET",url,"Connection: close\r\n")) {
-    output = client.body();
+    output = client.textBody();
     return true;
     }
   else {
@@ -374,8 +291,8 @@ GMAudioScrobbler::GMAudioScrobbler(FXObject* tgt,FXSelector msg) :
   nsubmitted(0),
   nfailed(0) {
 
-  username=FXApp::instance()->reg().readStringEntry("LastFM","username",NULL);
-  password=FXApp::instance()->reg().readStringEntry("LastFM","password",NULL);
+  username=FXApp::instance()->reg().readStringEntry("LastFM","username",nullptr);
+  password=FXApp::instance()->reg().readStringEntry("LastFM","password",nullptr);
 
   FXString uri = FXApp::instance()->reg().readStringEntry("LastFM","handshake-url",LASTFM_URL);
   if (uri==LASTFM_OLD_URL) {
@@ -390,7 +307,7 @@ GMAudioScrobbler::GMAudioScrobbler(FXObject* tgt,FXSelector msg) :
 
   /// Get last session
   if (mode==SERVICE_LASTFM) {
-    session=FXApp::instance()->reg().readStringEntry("LastFM","session",NULL);
+    session=FXApp::instance()->reg().readStringEntry("LastFM","session",nullptr);
     if (!session.empty()) {
       FXTRACE((60,"GMAudioScrobbler::GMAudioScrobbler - Session: %s\n",session.text()));
       nowplaying_url=handshake_url;
@@ -431,7 +348,7 @@ GMAudioScrobbler::~GMAudioScrobbler(){
  ******************************************************************************/
 
 FXuint GMAudioScrobbler::getService() {
-  FXString host = GMURL::host(handshake_url);
+  FXString host = FXURL::host(handshake_url);
   if (host=="ws.audioscrobbler.com")
     return SERVICE_LASTFM;
   else if (host=="turtle.libre.fm")
@@ -490,7 +407,7 @@ void GMAudioScrobbler::login(const FXString & user,const FXString & pass) {
   else {
     if (mode==SERVICE_LIBREFM) {
       FXString newpass=pass;
-      checksum(newpass);
+      ap_md5_digest(newpass);
       if (user!=username || newpass!=password) {
         username=user;
         password=newpass;
@@ -637,23 +554,23 @@ void GMAudioScrobbler::enable() {
 
 
 FXString GMAudioScrobbler::getUsername() {
-  FXMutexLock lock(mutex_data);
+  FXScopedMutex lock(mutex_data);
   return username;
   }
 
 FXbool GMAudioScrobbler::hasPassword(){
-  FXMutexLock lock(mutex_data);
+  FXScopedMutex lock(mutex_data);
   return !password.empty();
   }
 
 
 FXbool GMAudioScrobbler::isBanned(){
-  FXMutexLock lock(mutex_data);
+  FXScopedMutex lock(mutex_data);
   return (flags&FLAG_BANNED);
   }
 
 FXbool GMAudioScrobbler::isEnabled(){
-  FXMutexLock lock(mutex_data);
+  FXScopedMutex lock(mutex_data);
   return !(flags&FLAG_DISABLED);
   }
 
@@ -667,7 +584,7 @@ FXbool GMAudioScrobbler::isEnabled(){
 void GMAudioScrobbler::load_queue(){
   FXTRACE((60,"GMAudioScrobbler::load_queue\n"));
   FXuint version,size;
-  FXString filename = FXSystem::getHomeDirectory() + SCROBBLER_CACHE_FILE;
+  FXString filename = GMApp::getCacheDirectory() + SCROBBLER_CACHE_FILE;
   FXFileStream store;
   if (store.open(filename,FXStreamLoad)){
     store >> version;
@@ -685,19 +602,21 @@ void GMAudioScrobbler::load_queue(){
 
 
 void GMAudioScrobbler::save_queue(){
-  FXTRACE((60,"GMAudioScrobbler::save_queue => %d entries\n",submitqueue.no()));
-  FXuint version=20080501;
-  FXString filename = FXSystem::getHomeDirectory() + SCROBBLER_CACHE_FILE;
+  FXTRACE((60,"GMAudioScrobbler::save_queue => %ld entries\n",submitqueue.no()));
+  FXuint version=20080501,size;
   if (submitqueue.no()) {
+    FXString filename = GMApp::getCacheDirectory(true) + SCROBBLER_CACHE_FILE;
     FXFileStream store;
     if (store.open(filename,FXStreamSave)){
       store << version;
-      store << submitqueue.no();
+      size = submitqueue.no();
+      store << size;
       for (FXint i=0;i<submitqueue.no();i++)
         submitqueue[i].save(store);
       }
     }
   else {
+    FXString filename = GMApp::getCacheDirectory() + SCROBBLER_CACHE_FILE;
     FXFile::remove(filename);
     }
   }
@@ -705,18 +624,10 @@ void GMAudioScrobbler::save_queue(){
 FXbool GMAudioScrobbler::waitForTask() {
   FXTRACE((60,"GMAudioScrobbler::waitForTask\n"));
   if (timeout>0) {
-#if FOXVERSION < FXVERSION(1,7,0)
-    FXlong wakeuptime = FXThread::time()+((FXlong)timeout*1000000000LL); // absolute time
-#else
     FXlong wakeuptime = ((FXlong)timeout*1000000000LL); // relative time
-    FXlong start      = FXThread::time();
-#endif
+    FXlong starttime  = FXThread::time();
     while(1) {
-#if FOXVERSION < FXVERSION(1,7,0)
-      FXTRACE((60,"GMAudioScrobbler::waitForTask => %ld\n",timeout));
-#else
-      FXTRACE((60,"GMAudioScrobbler::waitForTask => %ld\n",wakeuptime));
-#endif
+      FXTRACE((60,"GMAudioScrobbler::waitForTask => %lld\n",wakeuptime/1000000000LL));
       mutex_task.lock();
       if (!condition_task.wait(mutex_task,wakeuptime)){
         mutex_task.unlock();
@@ -724,7 +635,7 @@ FXbool GMAudioScrobbler::waitForTask() {
         }
       else  {
         mutex_task.unlock();
-        FXMutexLock lock(mutex_data);
+        FXScopedMutex lock(mutex_data);
 
         if (flags&FLAG_SHUTDOWN)
           return false;
@@ -734,15 +645,13 @@ FXbool GMAudioScrobbler::waitForTask() {
           return true;
           }
 
-#if FOXVERSION > FXVERSION(1,7,0)
         FXlong now  = FXThread::time();
-        wakeuptime -= (now - start);
-        start       = now;
+        wakeuptime -= (now - starttime);
+        starttime   = now;
 
         /// Done waiting
         if (wakeuptime<=0)
           return true;
-#endif
         FXTRACE((60,"GMAudioScrobbler::waitForTask => reset\n"));
         }
       }
@@ -757,7 +666,7 @@ FXbool GMAudioScrobbler::waitForTask() {
 
 FXuchar GMAudioScrobbler::getNextTask() {
   FXTRACE((60,"GMAudioScrobbler::getNextTask\n"));
-  FXMutexLock lock(mutex_data);
+  FXScopedMutex lock(mutex_data);
 
   if (flags&FLAG_SHUTDOWN){
     flags&=~FLAG_SHUTDOWN;
@@ -815,6 +724,8 @@ FXuchar GMAudioScrobbler::getNextTask() {
 FXint GMAudioScrobbler::run() {
   FXTRACE((60,"GMAudioScrobbler::run\n"));
 
+  ap_set_thread_name("gm_scrobbler");
+
   FXuchar next=TASK_NONE;
   do {
     while((next=getNextTask())!=TASK_NONE) {
@@ -868,30 +779,30 @@ void GMAudioScrobbler::set_submit_failed() {
 
 
 void GMAudioScrobbler::create_token_request(FXString & request) {
-  FXMutexLock lock(mutex_data);
+  FXScopedMutex lock(mutex_data);
   FXTRACE((60,"GMAudioScrobbler::create_token_request\n"));
-  FXString signature="api_key"CLIENT_KEY"methodauth.getToken"CLIENT_SECRET;
-  checksum(signature);
-  request=GMStringFormat("method=auth.getToken&api_key="CLIENT_KEY"&api_sig=%s",signature.text());
+  FXString signature="api_key" CLIENT_KEY "methodauth.getToken" CLIENT_SECRET;
+  ap_md5_digest(signature);
+  request=FXString::value("method=auth.getToken&api_key=" CLIENT_KEY "&api_sig=%s",signature.text());
   flags&=~(FLAG_LOGIN_CHANGED);
   }
 
 
 void GMAudioScrobbler::process_token_response(const FXString & response){
-  FXMutexLock lock(mutex_data);
+  FXScopedMutex lock(mutex_data);
   if (flags&FLAG_LOGIN_CHANGED) return;
-  ServiceResponse service;
+  ServiceResponse sr;
   FXTRACE((60,"GMAudioScrobbler::process_token_response\n%s\n",response.text()));
-  if (service.parse(response) && service.getStatus()) {
-    token=service.getToken();
+  if (sr.parse(response) && sr.getStatus()) {
+    token=sr.getToken();
     FXTRACE((60,"GMAudioScrobbler::process_token_response => token=%s\n",token.text()));
-    FXString url="http://www.last.fm/api/auth/?api_key="CLIENT_KEY"&token="+token;
-    feedback.message(target,FXSEL(SEL_OPENED,message),url.text(),url.length());
+    FXString url="http://www.last.fm/api/auth/?api_key=" CLIENT_KEY "&token="+token;
+    feedback.message(target, FXSEL(SEL_OPENED,message), url.text(), url.length() + 1);
     reset_timeout(); /// Reset timer
     set_timeout();   /// Let's wait at least 60s
     }
   else {
-    FXTRACE((60,"last.fm service failed with code %d: %s\n",service.getErrorCode(),service.getErrorMessage().text()));
+    FXTRACE((60,"last.fm service failed with code %d: %s\n",sr.getErrorCode(),sr.getErrorMessage().text()));
     flags|=FLAG_BADAUTH;
     }
   }
@@ -899,19 +810,19 @@ void GMAudioScrobbler::process_token_response(const FXString & response){
 
 
 FXuint GMAudioScrobbler::create_handshake_request(FXString & request) {
-  FXMutexLock lock(mutex_data);
+  FXScopedMutex lock(mutex_data);
   FXTRACE((60,"GMAudioScrobbler::create_handshake_request\n"));
   if (mode==SERVICE_LASTFM) {
-    FXString signature=GMStringFormat("api_key%smethodauth.getSessiontoken%s%s",CLIENT_KEY,token.text(),CLIENT_SECRET);
-    checksum(signature);
-    request=GMStringFormat("method=auth.getSession&api_key=%s&api_sig=%s&token=%s",CLIENT_KEY,signature.text(),token.text());
+    FXString signature=FXString::value("api_key%smethodauth.getSessiontoken%s%s",CLIENT_KEY,token.text(),CLIENT_SECRET);
+    ap_md5_digest(signature);
+    request=FXString::value("method=auth.getSession&api_key=%s&api_sig=%s&token=%s",CLIENT_KEY,signature.text(),token.text());
     }
   else {
     FXlong timestamp = FXThread::time()/1000000000;
-    FXString timestamp_text = GMStringVal(timestamp);
+    FXString timestamp_text = FXString::value(timestamp);
     FXString tk = password + timestamp_text;
-    checksum(tk);
-    request=GMStringFormat("/?hs=true&p=1.2&c="CLIENT_ID"&v="CLIENT_VERSION"&u=%s&t=%s&a=%s",username.text(),timestamp_text.text(),tk.text());
+    ap_md5_digest(tk);
+    request=FXString::value("/?hs=true&p=1.2&c=" CLIENT_ID "&v=" CLIENT_VERSION "&u=%s&t=%s&a=%s",username.text(),timestamp_text.text(),tk.text());
     }
   flags&=~(FLAG_LOGIN_CHANGED);
   return mode;
@@ -921,15 +832,14 @@ FXuint GMAudioScrobbler::create_handshake_request(FXString & request) {
 
 
 void GMAudioScrobbler::process_handshake_response(const FXString & response){
-  FXMutexLock lock(mutex_data);
+  FXScopedMutex lock(mutex_data);
   if (flags&FLAG_LOGIN_CHANGED) return;
-  FXTRACE((60,"GMAudioScrobbler::process_handshake_response\n%s",response.text()));
-
+  FXTRACE((60,"GMAudioScrobbler::process_handshake_response\n%s\n",response.text()));
   if (mode==SERVICE_LASTFM) {
-    ServiceResponse service;
-    if (!service.parse(response) || !service.getStatus()){
-      FXTRACE((60,"last.fm service failed with code %d: %s\n",service.getErrorCode(),service.getErrorMessage().text()));
-      switch(service.getErrorCode()) {
+    ServiceResponse sr;
+    if (!sr.parse(response) || !sr.getStatus()){
+      FXTRACE((60,"last.fm service failed with code %d: %s\n",sr.getErrorCode(),sr.getErrorMessage().text()));
+      switch(sr.getErrorCode()) {
         case LASTFM_ERROR_TOKEN_EXPIRED     : token.clear();        break;
         case LASTFM_ERROR_TOKEN_UNAUTHORIZED:
         case LASTFM_ERROR_OFFLINE           :
@@ -938,7 +848,7 @@ void GMAudioScrobbler::process_handshake_response(const FXString & response){
         }
       }
     else {
-      session=service.getSessionKey();
+      session=sr.getSessionKey();
       nowplaying_url=handshake_url;
       submit_url=handshake_url;
       flags&=~FLAG_BADAUTH;
@@ -1043,11 +953,11 @@ void GMAudioScrobbler::loveban() {
 
 
 void GMAudioScrobbler::create_nowplaying_request(FXString & request) {
-  FXMutexLock lock(mutex_data);
+  FXScopedMutex lock(mutex_data);
   FXTRACE((60,"GMAudioScrobbler::create_nowplaying_request\n"));
   if (mode==SERVICE_LASTFM) {
-    FXString signature=GMStringFormat("album%s"
-                                      "api_key"CLIENT_KEY
+    FXString signature=FXString::value("album%s"
+                                      "api_key" CLIENT_KEY
                                       "artist%s"
                                       "duration%d"
                                       "methodtrack.updateNowPlaying"
@@ -1062,15 +972,15 @@ void GMAudioScrobbler::create_nowplaying_request(FXString & request) {
                                       nowplayingtrack.title.text(),
                                       nowplayingtrack.no);
 
-    checksum(signature);
+    ap_md5_digest(signature);
 
-    request=GMStringFormat("method=track.updateNowPlaying"
+    request=FXString::value("method=track.updateNowPlaying"
                            "&track=%s"
                            "&artist=%s"
                            "&album=%s"
                            "&trackNumber=%d"
                            "&duration=%d"
-                           "&api_key="CLIENT_KEY
+                           "&api_key=" CLIENT_KEY
                            "&api_sig=%s"
                            "&sk=%s",
                            gm_url_encode(nowplayingtrack.title).text(),
@@ -1082,7 +992,7 @@ void GMAudioScrobbler::create_nowplaying_request(FXString & request) {
                            session.text());
     }
   else {
-    request=GMStringFormat("s=%s&a=%s&t=%s&b=%s&l=%d&n=%d&m",gm_url_encode(session).text(),
+    request=FXString::value("s=%s&a=%s&t=%s&b=%s&l=%d&n=%d&m",gm_url_encode(session).text(),
                                                              gm_url_encode(nowplayingtrack.artist).text(),
                                                              gm_url_encode(nowplayingtrack.title).text(),
                                                              gm_url_encode(nowplayingtrack.album).text(),
@@ -1094,14 +1004,15 @@ void GMAudioScrobbler::create_nowplaying_request(FXString & request) {
 
 
 void GMAudioScrobbler::process_nowplaying_response(const FXString & response){
-  FXMutexLock lock(mutex_data);
+  FXScopedMutex lock(mutex_data);
   if (flags&FLAG_LOGIN_CHANGED) return;
-  FXTRACE((60,"GMAudioScrobbler::process_nowplaying_response:\n %s\n",response.text()));
+  FXTRACE((60,"GMAudioScrobbler::process_nowplaying_response:\n%s\n",response.text()));
   if (mode==SERVICE_LASTFM) {
-    ServiceResponse service;
-    if (!service.parse(response) || !service.getStatus()) {
-      FXTRACE((60,"last.fm service failed with code %d: %s\n",service.getErrorCode(),service.getErrorMessage().text()));
-      switch(service.getErrorCode()) {
+    ServiceResponse sr;
+    if (!sr.parse(response) || !sr.getStatus()) {
+      FXASSERT(0);
+      FXTRACE((60,"last.fm service failed with code %d: %s\n",sr.getErrorCode(),sr.getErrorMessage().text()));
+      switch(sr.getErrorCode()) {
         case LASTFM_ERROR_UNKNOWN      :
         case LASTFM_ERROR_OFFLINE      :
         case LASTFM_ERROR_UNAVAILABLE  : set_timeout();           break;
@@ -1113,7 +1024,6 @@ void GMAudioScrobbler::process_nowplaying_response(const FXString & response){
       }
     }
   else {
-    FXString response;
     FXString code=response.section('\n',0);
     FXTRACE((70,"Now Playing Response: %s\n\n",response.text()));
     if (compare(code,"BADSESSION",10)==0) {
@@ -1124,7 +1034,7 @@ void GMAudioScrobbler::process_nowplaying_response(const FXString & response){
 
 
 void GMAudioScrobbler::create_submit_request(FXString & request) {
-  FXMutexLock lock(mutex_data);
+  FXScopedMutex lock(mutex_data);
   FXTRACE((60,"GMAudioScrobbler::create_submit_request\n"));
   FXint i,s;
   FXint ntracks = FXMIN(50,submitqueue.no());
@@ -1132,8 +1042,8 @@ void GMAudioScrobbler::create_submit_request(FXString & request) {
 
   if (mode==SERVICE_LASTFM) {
     if (ntracks==1) {
-      signature=GMStringFormat("album[0]%s"
-                               "api_key"CLIENT_KEY
+      signature=FXString::value("album[0]%s"
+                               "api_key" CLIENT_KEY
                                "artist[0]%s"
                                "duration[0]%d"
                                "methodtrack.scrobble"
@@ -1153,27 +1063,27 @@ void GMAudioScrobbler::create_submit_request(FXString & request) {
     else if (ntracks<10) {
 
       for (i=0;i<ntracks;i++)
-        signature+=GMStringFormat("album[%d]%s",i,submitqueue[i].album.text());
+        signature+=FXString::value("album[%d]%s",i,submitqueue[i].album.text());
 
-      signature+="api_key"CLIENT_KEY;
+      signature+="api_key" CLIENT_KEY;
 
       for (i=0;i<ntracks;i++)
-        signature+=GMStringFormat("artist[%d]%s",i,submitqueue[i].artist.text());
+        signature+=FXString::value("artist[%d]%s",i,submitqueue[i].artist.text());
 
       for (i=0;i<ntracks;i++)
-        signature+=GMStringFormat("duration[%d]%d",i,submitqueue[i].duration);
+        signature+=FXString::value("duration[%d]%d",i,submitqueue[i].duration);
 
       signature+="methodtrack.scrobble";
       signature+="sk"+session;
 
       for (i=0;i<ntracks;i++)
-        signature+=GMStringFormat("timestamp[%d]%u",i,submitqueue[i].getTimeStamp());
+        signature+=FXString::value("timestamp[%d]%u",i,submitqueue[i].getTimeStamp());
 
       for (i=0;i<ntracks;i++)
-        signature+=GMStringFormat("trackNumber[%d]%d",i,submitqueue[i].no);
+        signature+=FXString::value("trackNumber[%d]%d",i,submitqueue[i].no);
 
       for (i=0;i<ntracks;i++)
-        signature+=GMStringFormat("track[%d]%s",i,submitqueue[i].title.text());
+        signature+=FXString::value("track[%d]%s",i,submitqueue[i].title.text());
 
       signature+=CLIENT_SECRET;
       }
@@ -1181,37 +1091,37 @@ void GMAudioScrobbler::create_submit_request(FXString & request) {
       /// albums
       for (s=0,i=10;i<ntracks;i++){
         if ((i%10)==0) {
-          signature+=GMStringFormat("album[%d]%s",s,submitqueue[s].album.text());
+          signature+=FXString::value("album[%d]%s",s,submitqueue[s].album.text());
           s++;
           }
-        signature+=GMStringFormat("album[%d]%s",i,submitqueue[i].album.text());
+        signature+=FXString::value("album[%d]%s",i,submitqueue[i].album.text());
         }
       for (;s<10;s++)
-        signature+=GMStringFormat("album[%d]%s",s,submitqueue[s].album.text());
+        signature+=FXString::value("album[%d]%s",s,submitqueue[s].album.text());
 
-      signature+="api_key"CLIENT_KEY;
+      signature+="api_key" CLIENT_KEY;
 
       /// artists
       for (s=0,i=10;i<ntracks;i++){
         if ((i%10)==0) {
-          signature+=GMStringFormat("artist[%d]%s",s,submitqueue[s].artist.text());
+          signature+=FXString::value("artist[%d]%s",s,submitqueue[s].artist.text());
           s++;
           }
-        signature+=GMStringFormat("artist[%d]%s",i,submitqueue[i].artist.text());
+        signature+=FXString::value("artist[%d]%s",i,submitqueue[i].artist.text());
         }
       for (;s<10;s++)
-        signature+=GMStringFormat("artist[%d]%s",s,submitqueue[s].artist.text());
+        signature+=FXString::value("artist[%d]%s",s,submitqueue[s].artist.text());
 
       /// duration
       for (s=0,i=10;i<ntracks;i++){
         if ((i%10)==0) {
-          signature+=GMStringFormat("duration[%d]%d",s,submitqueue[s].duration);
+          signature+=FXString::value("duration[%d]%d",s,submitqueue[s].duration);
           s++;
           }
-        signature+=GMStringFormat("duration[%d]%d",i,submitqueue[i].duration);
+        signature+=FXString::value("duration[%d]%d",i,submitqueue[i].duration);
         }
       for (;s<10;s++)
-        signature+=GMStringFormat("duration[%d]%d",s,submitqueue[s].duration);
+        signature+=FXString::value("duration[%d]%d",s,submitqueue[s].duration);
 
       signature+="methodtrack.scrobble";
       signature+="sk"+session;
@@ -1219,46 +1129,46 @@ void GMAudioScrobbler::create_submit_request(FXString & request) {
       /// Timestamp
       for (s=0,i=10;i<ntracks;i++){
         if ((i%10)==0) {
-          signature+=GMStringFormat("timestamp[%d]%u",s,submitqueue[s].getTimeStamp());
+          signature+=FXString::value("timestamp[%d]%u",s,submitqueue[s].getTimeStamp());
           s++;
           }
-        signature+=GMStringFormat("timestamp[%d]%u",i,submitqueue[i].getTimeStamp());
+        signature+=FXString::value("timestamp[%d]%u",i,submitqueue[i].getTimeStamp());
         }
       for (;s<10;s++)
-        signature+=GMStringFormat("timestamp[%d]%u",s,submitqueue[s].getTimeStamp());
+        signature+=FXString::value("timestamp[%d]%u",s,submitqueue[s].getTimeStamp());
 
       /// track number
       for (s=0,i=10;i<ntracks;i++){
         if ((i%10)==0) {
-          signature+=GMStringFormat("trackNumber[%d]%d",s,submitqueue[s].no);
+          signature+=FXString::value("trackNumber[%d]%d",s,submitqueue[s].no);
           s++;
           }
-        signature+=GMStringFormat("trackNumber[%d]%d",i,submitqueue[i].no);
+        signature+=FXString::value("trackNumber[%d]%d",i,submitqueue[i].no);
         }
       for (;s<10;s++)
-        signature+=GMStringFormat("trackNumber[%d]%d",s,submitqueue[s].no);
+        signature+=FXString::value("trackNumber[%d]%d",s,submitqueue[s].no);
 
 
       /// track
       for (s=0,i=10;i<ntracks;i++){
         if ((i%10)==0) {
-          signature+=GMStringFormat("track[%d]%s",s,submitqueue[s].title.text());
+          signature+=FXString::value("track[%d]%s",s,submitqueue[s].title.text());
           s++;
           }
-        signature+=GMStringFormat("track[%d]%s",i,submitqueue[i].title.text());
+        signature+=FXString::value("track[%d]%s",i,submitqueue[i].title.text());
         }
       for (;s<10;s++)
-        signature+=GMStringFormat("track[%d]%s",s,submitqueue[s].title.text());
+        signature+=FXString::value("track[%d]%s",s,submitqueue[s].title.text());
 
 
       signature+=CLIENT_SECRET;
       }
 
-    checksum(signature);
+    ap_md5_digest(signature);
 
     request="method=track.scrobble";
     for (i=0;i<ntracks;i++) {
-      request+=GMStringFormat("&track[%d]=%s"
+      request+=FXString::value("&track[%d]=%s"
                               "&timestamp[%d]=%u"
                               "&artist[%d]=%s"
                               "&album[%d]=%s"
@@ -1271,13 +1181,13 @@ void GMAudioScrobbler::create_submit_request(FXString & request) {
                               i,submitqueue[i].no,
                               i,submitqueue[i].duration);
       }
-    request+=GMStringFormat("&api_key="CLIENT_KEY"&api_sig=%s&sk=%s",signature.text(),session.text());
+    request+=FXString::value("&api_key=" CLIENT_KEY "&api_sig=%s&sk=%s",signature.text(),session.text());
     }
   else {
     request+="s=";
     request+=gm_url_encode(session);
     for (i=0;i<ntracks;i++) {
-      request+=GMStringFormat("&a[%d]=%s"
+      request+=FXString::value("&a[%d]=%s"
                               "&t[%d]=%s"
                               "&i[%d]=%u"
                               "&o[%d]=P"
@@ -1300,13 +1210,14 @@ void GMAudioScrobbler::create_submit_request(FXString & request) {
   }
 
 void GMAudioScrobbler::process_submit_response(const FXString & response){
-  FXMutexLock lock(mutex_data);
-  FXTRACE((60,"GMAudioScrobbler::process_submit_response\n"));
+  FXScopedMutex lock(mutex_data);
+  FXTRACE((60,"GMAudioScrobbler::process_submit_response\n%s\n",response.text()));
   if (mode==SERVICE_LASTFM) {
-    ServiceResponse service;
-    if (!service.parse(response) || !service.getStatus()) {
-      FXTRACE((60,"last.fm service failed with code %d: %s\n",service.getErrorCode(),service.getErrorMessage().text()));
-      switch(service.getErrorCode()) {
+    ServiceResponse sr;
+    if (!sr.parse(response) || !sr.getStatus()) {
+      FXASSERT(0);
+      FXTRACE((60,"last.fm service failed with code %d: %s\n",sr.getErrorCode(),sr.getErrorMessage().text()));
+      switch(sr.getErrorCode()) {
         case LASTFM_ERROR_UNKNOWN      :
         case LASTFM_ERROR_OFFLINE      :
         case LASTFM_ERROR_UNAVAILABLE  : set_timeout();           break;
@@ -1353,11 +1264,11 @@ void GMAudioScrobbler::process_submit_response(const FXString & response){
 
 /* LastFM */
 void GMAudioScrobbler::create_loveban_request(FXString & request){
-  FXMutexLock lock(mutex_data);
+  FXScopedMutex lock(mutex_data);
   FXTRACE((60,"GMAudioScrobbler::create_loveban_request\n"));
   FXASSERT(mode==SERVICE_LASTFM);
 
-  FXString signature=GMStringFormat("api_key"CLIENT_KEY
+  FXString signature=FXString::value("api_key" CLIENT_KEY
                            "artist%s"
                            "methodtrack.love" //methodtrack.ban
                            "sk%s"
@@ -1367,12 +1278,12 @@ void GMAudioScrobbler::create_loveban_request(FXString & request){
                            session.text(),
                            submitqueue[0].title.text());
 
-  checksum(signature);
+  ap_md5_digest(signature);
 
-  request=GMStringFormat("method=track.love" ////method=track.ban
+  request=FXString::value("method=track.love" ////method=track.ban
                          "&track=%s"
                          "&artist=%s"
-                         "&api_key="CLIENT_KEY
+                         "&api_key=" CLIENT_KEY
                          "&api_sig=%s"
                          "&sk=%s",
                          gm_url_encode(submitqueue[0].title).text(),
@@ -1382,11 +1293,11 @@ void GMAudioScrobbler::create_loveban_request(FXString & request){
   }
 
 void GMAudioScrobbler::process_loveban_response(const FXString & response){
-  FXMutexLock lock(mutex_data);
-  FXTRACE((60,"GMAudioScrobbler::process_loveban_response\n"));
-  ServiceResponse service;
-  if (!service.parse(response) || !service.getStatus()) {
+  FXScopedMutex lock(mutex_data);
+  FXTRACE((60,"GMAudioScrobbler::process_loveban_response\n%s\n",response.text()));
+  ServiceResponse sr;
+  if (!sr.parse(response) || !sr.getStatus()) {
     session.clear();
-    FXTRACE((60,"last.fm service failed with code %d: %s\n",service.getErrorCode(),service.getErrorMessage().text()));
+    FXTRACE((60,"last.fm service failed with code %d: %s\n",sr.getErrorCode(),sr.getErrorMessage().text()));
     }
   }
