@@ -1,7 +1,7 @@
 /*******************************************************************************
 *                         Goggles Music Manager                                *
 ********************************************************************************
-*           Copyright (C) 2007-2011 by Sander Jansen. All Rights Reserved      *
+*           Copyright (C) 2007-2021 by Sander Jansen. All Rights Reserved      *
 *                               ---                                            *
 * This program is free software: you can redistribute it and/or modify         *
 * it under the terms of the GNU General Public License as published by         *
@@ -47,60 +47,36 @@ private:
   FXMutex mutex;
   FXHash  tm;
   FXHash  connections;
-#if FOXVERSION < FXVERSION(1,7,0)
-  FXHash  watches;
-#endif
 public:
   GMDBusGlobal() {
     dbus_threads_init_default();
     }
 
   GMDBus * find(DBusConnection* dc) {
-    FXMutexLock lock(mutex);
-    return reinterpret_cast<GMDBus*>(connections.find(dc));
+    FXScopedMutex lock(mutex);
+    return static_cast<GMDBus*>(connections.at(dc));
     }
 
   void setuphooks() {
-#if FOXVERSION < FXVERSION(1,7,0)
-    for (FXint i=0;i<connections.size();i++) {
-#else
-    for (FXuint i=0;i<connections.size();i++) {
-#endif
+    for (FXint i=0;i<connections.no();i++) {
       if (!connections.empty(i)) {
-        ((GMDBus*)connections.value(i))->setup_event_loop();
+        static_cast<GMDBus*>(connections.data(i))->setup_event_loop();
         }
       }
     }
 
   void insert(DBusConnection * dc,GMDBus * fxdc) {
-    FXMutexLock lock(mutex);
+    FXScopedMutex lock(mutex);
     connections.insert(dc,fxdc);
     }
 
   void remove(DBusConnection * dc) {
-    FXMutexLock lock(mutex);
+    FXScopedMutex lock(mutex);
     connections.remove(dc);
     }
 
-#if FOXVERSION < FXVERSION(1,7,0)
-  DBusWatch * find(FXint fd) {
-    FXMutexLock lock(mutex);
-    return (DBusWatch*)watches.find((void*)(FXival)fd);
-    }
-
-  void insert(FXint fd,DBusWatch * watch){
-    FXMutexLock lock(mutex);
-    watches.insert((void*)(FXival)fd,watch);
-    }
-
-  void remove(FXint fd){
-    FXMutexLock lock(mutex);
-    watches.remove((void*)(FXival)fd);
-    }
-#endif
-
   GMDBusTimeout * find(DBusTimeout*t) {
-    return reinterpret_cast<GMDBusTimeout*>(tm.find(t));
+    return static_cast<GMDBusTimeout*>(tm.at(t));
     }
 
   void insert(DBusTimeout*t,GMDBusTimeout*f) {
@@ -117,10 +93,10 @@ static GMDBusGlobal fxdbus;
 class GMDBusTimeout : public FXObject {
 FXDECLARE(GMDBusTimeout);
 protected:
-  DBusTimeout* timeout;
-  FXuchar      flags;
+  DBusTimeout* timeout = nullptr;
+  FXuchar      flags = 0;
 protected:
-  GMDBusTimeout();
+  GMDBusTimeout(){}
 private:
   GMDBusTimeout(const GMDBusTimeout*);
   GMDBusTimeout& operator=(const GMDBusTimeout&);
@@ -149,10 +125,7 @@ FXIMPLEMENT(GMDBusTimeout,FXObject,GMDBusTimeoutMap,ARRAYNUMBER(GMDBusTimeoutMap
 
 
 
-GMDBusTimeout::GMDBusTimeout() {
-  }
-
-GMDBusTimeout::GMDBusTimeout(DBusTimeout *t) : timeout(t),flags(0) {
+GMDBusTimeout::GMDBusTimeout(DBusTimeout *t) : timeout(t){
   fxdbus.insert(timeout,this);
   }
 
@@ -175,10 +148,10 @@ void GMDBusTimeout::remove() {
   }
 
 
-static dbus_bool_t fxdbus_addtimeout(DBusTimeout *timeout,void */*ptr*/) {
-//  fxmessage("fxdbus_addtimeout %p\n",timeout);
+static dbus_bool_t fxdbus_addtimeout(DBusTimeout *timeout,void *) {
+  //fxmessage("fxdbus_addtimeout %p\n",timeout);
   GMDBusTimeout * tm = fxdbus.find(timeout);
-  if (tm==NULL)
+  if (tm==nullptr)
     tm = new GMDBusTimeout(timeout);
 
   tm->add();
@@ -186,14 +159,14 @@ static dbus_bool_t fxdbus_addtimeout(DBusTimeout *timeout,void */*ptr*/) {
   }
 
 
-static void fxdbus_removetimeout(DBusTimeout *timeout,void * /*data*/) {
-//  fxmessage("fxdbus_removetimeout %p\n",timeout);
+static void fxdbus_removetimeout(DBusTimeout *timeout,void *) {
+  //fxmessage("fxdbus_removetimeout %p\n",timeout);
   GMDBusTimeout * tm = fxdbus.find(timeout);
   if (tm) tm->remove();
   }
 
-static void fxdbus_toggletimeout(DBusTimeout *timeout,void* data) {
-//  fxmessage("fxdbus_toggletimeout %p\n",timeout);
+static void fxdbus_toggletimeout(DBusTimeout *timeout,void*data) {
+  //fxmessage("fxdbus_toggletimeout %p\n",timeout);
   if (dbus_timeout_get_enabled(timeout) && dbus_timeout_get_interval(timeout)>0)
     fxdbus_addtimeout(timeout,data);
   else
@@ -201,12 +174,12 @@ static void fxdbus_toggletimeout(DBusTimeout *timeout,void* data) {
   }
 
 long GMDBusTimeout::onTimeout(FXObject*,FXSelector,void*){
-//  fxmessage("onTimeout() %p {\n",timeout);
+  //fxmessage("onTimeout() %p {\n",timeout);
   flags|=FLAG_CALLBACK;
   dbus_timeout_handle(timeout);
   if (flags&FLAG_REMOVED) {
     delete this;
-    fxmessage("}\n");
+    //fxmessage("}\n");
     return 1;
     }
   else {
@@ -215,7 +188,7 @@ long GMDBusTimeout::onTimeout(FXObject*,FXSelector,void*){
       }
     }
   flags&=~FLAG_CALLBACK;
-//  fxmessage("}\n");
+  //fxmessage("}\n");
   return 1;
   }
 
@@ -235,7 +208,7 @@ long GMDBusTimeout::onTimeout(FXObject*,FXSelector,void*){
 
 static dbus_bool_t fxdbus_addwatch(DBusWatch *watch,void * data) {
   FXTRACE((35,"fxdbus_addwatch()\n"));
-  GMDBus * dc = reinterpret_cast<GMDBus*>(data);
+  GMDBus * dc = static_cast<GMDBus*>(data);
   FXuint mode  = INPUT_EXCEPT;
   FXuint flags = dbus_watch_get_flags(watch);
 
@@ -248,19 +221,9 @@ static dbus_bool_t fxdbus_addwatch(DBusWatch *watch,void * data) {
       mode|=INPUT_WRITE;
 
 #if DBUSVERSION == MKDBUSVERSION(1,1,20) || DBUSVERSION >= MKDBUSVERSION(1,2,0)
-#if FOXVERSION < FXVERSION(1,7,0)
-  fxdbus.insert(dbus_watch_get_unix_fd(watch),watch);
-  return FXApp::instance()->addInput((FXInputHandle)dbus_watch_get_unix_fd(watch),mode,dc,GMDBus::ID_HANDLE);
-#else
   return FXApp::instance()->addInput(dc,GMDBus::ID_HANDLE,(FXInputHandle)dbus_watch_get_unix_fd(watch),mode,watch);
-#endif
-#else
-#if FOXVERSION < FXVERSION(1,7,0)
-  fxdbus.insert(dbus_watch_get_fd(watch),watch);
-  return FXApp::instance()->addInput((FXInputHandle)dbus_watch_get_fd(watch),mode,dc,GMDBus::ID_HANDLE);
 #else
   return FXApp::instance()->addInput(dc,GMDBus::ID_HANDLE,(FXInputHandle)dbus_watch_get_fd(watch),mode,watch);
-#endif
 #endif
 
   }
@@ -279,14 +242,8 @@ static void fxdbus_removewatch(DBusWatch *watch,void *) {
 */
   FXuint mode=INPUT_READ|INPUT_WRITE|INPUT_EXCEPT;
 #if DBUSVERSION == MKDBUSVERSION(1,1,20) || DBUSVERSION >= MKDBUSVERSION(1,2,0)
-#if FOXVERSION < FXVERSION(1,7,0)
-  fxdbus.remove(dbus_watch_get_unix_fd(watch));
-#endif
   FXApp::instance()->removeInput(dbus_watch_get_unix_fd(watch),mode);
 #else
-#if FOXVERSION < FXVERSION(1,7,0)
-  fxdbus.remove(dbus_watch_get_fd(watch));
-#endif
   FXApp::instance()->removeInput(dbus_watch_get_fd(watch),mode);
 #endif
 
@@ -319,14 +276,14 @@ FXDEFMAP(GMDBus) GMDBusMap[]={
 
 FXIMPLEMENT(GMDBus,FXObject,GMDBusMap,ARRAYNUMBER(GMDBusMap));
 
-GMDBus::GMDBus() : dc(NULL) {
+GMDBus::GMDBus() {
   }
 
 GMDBus::~GMDBus(){
   if (dc) {
     fxdbus.remove(dc);
     dbus_connection_unref(dc);
-    dc=NULL;
+    dc=nullptr;
     FXApp::instance()->removeChore(this,ID_DISPATCH);
     }
   }
@@ -342,12 +299,12 @@ void GMDBus::initEventLoop() {
 
 
 FXbool GMDBus::open(DBusBusType bustype/*=DBUS_BUS_SESSION*/){
-  FXASSERT(dc==NULL);
-  dc = dbus_bus_get(bustype,NULL);
-  if (dc==NULL) return false;
+  FXASSERT(dc==nullptr);
+  dc = dbus_bus_get(bustype,nullptr);
+  if (dc==nullptr) return false;
   if (fxdbus.find(dc)) {
     dbus_connection_unref(dc);
-    dc=NULL;
+    dc=nullptr;
     return false;
     }
   fxdbus.insert(dc,this);
@@ -379,7 +336,7 @@ FXString GMDBus::dbusversion() {
 #if (DBUSVERSION == MKDBUSVERSION(1,1,20)) || DBUSVERSION >= MKDBUSVERSION(1,2,0)
   int major,minor,micro;
   dbus_get_version(&major,&minor,&micro);
-  return GMStringFormat("%d.%d.%d",major,minor,micro);
+  return FXString::value("%d.%d.%d",major,minor,micro);
 #else
   return FXString("1.0.x");
 #endif
@@ -393,7 +350,7 @@ struct CallTarget{
 static void fxdbus_pendingcallfree(void *memory){
   //fxmessage("fxdbuspendingcallfree\n");
   if (memory){
-    CallTarget * call = (CallTarget*)memory;
+    CallTarget * call = static_cast<CallTarget*>(memory);
     delete call;
     }
   }
@@ -401,9 +358,9 @@ static void fxdbus_pendingcallfree(void *memory){
 static void fxdbus_pendingcallnotify(DBusPendingCall*pending,void*data){
   DBusMessage * msg = dbus_pending_call_steal_reply(pending);
   if (msg) {
-    CallTarget * call = (CallTarget*)data;
+    CallTarget * call = static_cast<CallTarget*>(data);
     if (call && call->target && call->message) {
-      call->target->handle(NULL,FXSEL(SEL_COMMAND,call->message),msg);
+      call->target->handle(nullptr,FXSEL(SEL_COMMAND,call->message),msg);
       }
     dbus_message_unref(msg);
     }
@@ -412,7 +369,7 @@ static void fxdbus_pendingcallnotify(DBusPendingCall*pending,void*data){
 
 FXbool GMDBus::sendWithReply(DBusMessage * msg,FXint timeout,FXObject*tgt,FXSelector sel){
   FXASSERT(msg);
-  DBusPendingCall * pending = NULL;
+  DBusPendingCall * pending = nullptr;
   if (dbus_connection_send_with_reply(dc,msg,&pending,timeout)) {
     if (pending) {
       CallTarget * call = new CallTarget;
@@ -446,37 +403,25 @@ void GMDBus::send(DBusMessage * msg,FXuint & serial){
 /*******************************************************************************************************/
 
 long GMDBus::onHandleRead(FXObject*,FXSelector,void*ptr){
-#if FOXVERSION < FXVERSION(1,7,0)
-  DBusWatch * watch = fxdbus.find((FXint)(FXival)ptr);
-#else
-  DBusWatch * watch = reinterpret_cast<DBusWatch*>(ptr);
-#endif
+  DBusWatch * watch = static_cast<DBusWatch*>(ptr);
   dbus_watch_handle(watch,DBUS_WATCH_READABLE);
   FXApp::instance()->addChore(this,ID_DISPATCH);
   return 0;
   }
 
 long GMDBus::onHandleWrite(FXObject*,FXSelector,void*ptr){
-#if FOXVERSION < FXVERSION(1,7,0)
-  DBusWatch * watch = fxdbus.find((FXint)(FXival)ptr);
-#else
-  DBusWatch * watch = reinterpret_cast<DBusWatch*>(ptr);
-#endif
+  DBusWatch * watch = static_cast<DBusWatch*>(ptr);
   dbus_watch_handle(watch,DBUS_WATCH_WRITABLE);
   return 0;
   }
 
 long GMDBus::onHandleExcept(FXObject*,FXSelector,void*ptr){
-#if FOXVERSION < FXVERSION(1,7,0)
-  DBusWatch * watch = fxdbus.find((FXint)(FXival)ptr);
-#else
-  DBusWatch * watch = reinterpret_cast<DBusWatch*>(ptr);
-#endif
+  DBusWatch * watch = static_cast<DBusWatch*>(ptr);
   dbus_watch_handle(watch,DBUS_WATCH_ERROR|DBUS_WATCH_HANGUP);
   return 0;
   }
 
-long GMDBus::onDispatch(FXObject*,FXSelector,void*/*ptr*/){
+long GMDBus::onDispatch(FXObject*,FXSelector,void*){
   if (dbus_connection_dispatch((DBusConnection*)dc)==DBUS_DISPATCH_DATA_REMAINS) {
     FXApp::instance()->addChore(this,ID_DISPATCH);
     }
@@ -495,15 +440,15 @@ void GMDBus::setup_event_loop() {
                                       fxdbus_addwatch,
                                       fxdbus_removewatch,
                                       fxdbus_togglewatch,
-                                      this,NULL);
+                                      this,nullptr);
 
   dbus_connection_set_timeout_functions(dc,
                                         fxdbus_addtimeout,
                                         fxdbus_removetimeout,
                                         fxdbus_toggletimeout,
-                                        this,NULL);
+                                        this,nullptr);
 
-  dbus_connection_set_wakeup_main_function(dc,fxdbus_wakeup,NULL,NULL);
+  dbus_connection_set_wakeup_main_function(dc,fxdbus_wakeup,nullptr,nullptr);
   }
 
 
@@ -511,7 +456,7 @@ void GMDBus::setup_event_loop() {
 
 
 static DBusHandlerResult dbus_proxy_filter(DBusConnection *,DBusMessage * msg,void * ptr){
-  GMDBusProxy * proxy = reinterpret_cast<GMDBusProxy*>(ptr);
+  GMDBusProxy * proxy = static_cast<GMDBusProxy*>(ptr);
   FXASSERT(proxy);
   FXuint type = dbus_message_get_type(msg);
 
@@ -529,19 +474,19 @@ static DBusHandlerResult dbus_proxy_filter(DBusConnection *,DBusMessage * msg,vo
 
     if (dbus_message_has_path(msg,DBUS_PATH_DBUS)) {
       if (dbus_message_is_signal(msg,DBUS_INTERFACE_DBUS,"NameOwnerChanged")) {
-        const FXchar * dbus_name=NULL;
-        const FXchar * new_owner=NULL;
-        const FXchar * old_owner=NULL;
-        if (dbus_message_get_args(msg,NULL,DBUS_TYPE_STRING,&dbus_name,DBUS_TYPE_STRING,&old_owner,DBUS_TYPE_STRING,&new_owner,DBUS_TYPE_INVALID)) {
+        const FXchar * dbus_name=nullptr;
+        const FXchar * new_owner=nullptr;
+        const FXchar * old_owner=nullptr;
+        if (dbus_message_get_args(msg,nullptr,DBUS_TYPE_STRING,&dbus_name,DBUS_TYPE_STRING,&old_owner,DBUS_TYPE_STRING,&new_owner,DBUS_TYPE_INVALID)) {
           if (compare(proxy->getName(),dbus_name)==0) {
-            if (new_owner==NULL || compare(new_owner,"")==0) {
-              proxy->handle(proxy,FXSEL(SEL_DESTROY,0),NULL);
+            if (new_owner==nullptr || compare(new_owner,"")==0) {
+              proxy->handle(proxy,FXSEL(SEL_DESTROY,0),nullptr);
               }
-            else if (old_owner==NULL || compare(old_owner,"")==0){
-              proxy->handle(proxy,FXSEL(SEL_CREATE,0),NULL);
+            else if (old_owner==nullptr || compare(old_owner,"")==0){
+              proxy->handle(proxy,FXSEL(SEL_CREATE,0),nullptr);
               }
             else {
-              proxy->handle(proxy,FXSEL(SEL_REPLACED,0),NULL);
+              proxy->handle(proxy,FXSEL(SEL_REPLACED,0),nullptr);
               }
             return DBUS_HANDLER_RESULT_HANDLED;
             }
@@ -557,6 +502,14 @@ static DBusHandlerResult dbus_proxy_filter(DBusConnection *,DBusMessage * msg,vo
   }
 
 
+
+struct GMDBusProxyReply {
+  FXObject * target;
+  FXSelector message;
+  GMDBusProxyReply(FXObject * t,FXSelector m) : target(t), message(m) {}
+  };
+
+
 FXDEFMAP(GMDBusProxy) GMDBusProxyMap[]={
   FXMAPFUNC(SEL_CREATE,   0,GMDBusProxy::onCreate),
   FXMAPFUNC(SEL_DESTROY,  0,GMDBusProxy::onDestroy),
@@ -568,33 +521,36 @@ FXDEFMAP(GMDBusProxy) GMDBusProxyMap[]={
 FXIMPLEMENT(GMDBusProxy,FXObject,GMDBusProxyMap,ARRAYNUMBER(GMDBusProxyMap));
 
 
-GMDBusProxy::GMDBusProxy() : target(NULL) {
+GMDBusProxy::GMDBusProxy() {
   }
 
 GMDBusProxy::~GMDBusProxy()  {
   FXString rule = "type='signal',sender='"+ name +"',path='" + path +"',interface='"+interface+"'";
-  dbus_bus_remove_match(bus->connection(),rule.text(),NULL);
+  dbus_bus_remove_match(bus->connection(),rule.text(),nullptr);
   dbus_connection_remove_filter(bus->connection(),dbus_proxy_filter,this);
+
+  /// remove any pending proxy replies;
+  for (FXint i=0;i<serial.no();i++) {
+    if (!serial.empty(i)) {
+      GMDBusProxyReply * reply = static_cast<GMDBusProxyReply*>(serial.data(i));
+      delete reply;
+      }
+    }
+  serial.clear();
   }
 
-GMDBusProxy::GMDBusProxy(GMDBus *c,const FXchar * n,const FXchar * p,const FXchar * i) : bus(c),name(n),path(p),interface(i),associated(true),target(NULL) {
+GMDBusProxy::GMDBusProxy(GMDBus *c,const FXchar * n,const FXchar * p,const FXchar * i) : bus(c),name(n),path(p),interface(i),associated(true),target(nullptr) {
   FXString rule = "type='signal',sender='"+ name +"',path='" + path +"',interface='"+interface+"'";
-  dbus_bus_add_match(bus->connection(),rule.text(),NULL);
-  dbus_connection_add_filter(bus->connection(),dbus_proxy_filter,this,NULL);
+  dbus_bus_add_match(bus->connection(),rule.text(),nullptr);
+  dbus_connection_add_filter(bus->connection(),dbus_proxy_filter,this,nullptr);
   }
 
 
-struct GMDBusProxyReply {
-  FXObject * target;
-  FXSelector message;
-  GMDBusProxyReply(FXObject * t,FXSelector m) : target(t), message(m) {}
-  };
-
 FXbool GMDBusProxy::matchSerial(DBusMessage * msg) {
   void * ptr;
   FXuint s=dbus_message_get_reply_serial(msg);
-  if (s && (ptr=serial.find((void*)(FXuval)s))!=NULL) {
-    GMDBusProxyReply * reply = reinterpret_cast<GMDBusProxyReply*>(ptr);
+  if (s && (ptr=serial.at((void*)(FXuval)s))!=nullptr) {
+    GMDBusProxyReply * reply = static_cast<GMDBusProxyReply*>(ptr);
     if (reply->target) reply->target->handle(this,FXSEL(SEL_COMMAND,reply->message),msg);
     serial.remove((void*)(FXuval)dbus_message_get_reply_serial(msg));
     delete reply;
@@ -614,10 +570,15 @@ void GMDBusProxy::send(DBusMessage*msg,FXObject * obj,FXSelector m) {
 
 
 
-DBusMessage * GMDBusProxy::method(const FXchar * method){
-  return dbus_message_new_method_call(name.text(),path.text(),interface.text(),method);
+DBusMessage * GMDBusProxy::method(const FXchar * methodcall){
+  return dbus_message_new_method_call(name.text(),path.text(),interface.text(),methodcall);
   }
 
+DBusMessage * GMDBusProxy::signal(const FXchar * sname){
+  return dbus_message_new_signal(path.text(),interface.text(),sname);
+  }
+
+
 long GMDBusProxy::onCreate(FXObject*,FXSelector,void*ptr) {
   associated=true;
   return target && target->tryHandle(this,FXSEL(SEL_DESTROY,message),ptr);
@@ -714,13 +675,23 @@ void gm_dbus_variant_append_string_list(DBusMessageIter * iter,const FXStringLis
   DBusMessageIter array;
   dbus_message_iter_open_container(iter,DBUS_TYPE_VARIANT,DBUS_TYPE_ARRAY_AS_STRING DBUS_TYPE_STRING_AS_STRING,&container);
   dbus_message_iter_open_container(&container,DBUS_TYPE_ARRAY,DBUS_TYPE_STRING_AS_STRING,&array);
-  for (FXint n=0;list.no();n++) {
-    dbus_message_iter_append_basic(&array,DBUS_TYPE_STRING,list[n].text());
+  for (FXint n=0;n<list.no();n++) {
+    gm_dbus_append_string(&array,list[n]);
     }
   dbus_message_iter_close_container(&container,&array);
   dbus_message_iter_close_container(iter,&container);
   }
 
+void gm_dbus_append_string(DBusMessageIter *iter,const FXString & value){
+  const FXchar * v = value.text();
+  dbus_message_iter_append_basic(iter,DBUS_TYPE_STRING,&v);
+  }
+
+
+void gm_dbus_append_string_pair(DBusMessageIter *iter,const FXchar * key,const FXchar * value){
+  dbus_message_iter_append_basic(iter,DBUS_TYPE_STRING,&key);
+  dbus_message_iter_append_basic(iter,DBUS_TYPE_STRING,&value);
+  }
 
 
 void gm_dbus_dict_append_int32(DBusMessageIter * iter,const FXchar * key,const FXint value){
@@ -805,18 +776,76 @@ void gm_dbus_dict_append_string_list(DBusMessageIter * iter,const FXchar * key,c
   gm_dbus_variant_append_string_list(&container,value);
   dbus_message_iter_close_container(iter,&container);
   }
-  
-void gm_dbus_append_string(DBusMessageIter *iter,const FXString & value){
-  const FXchar * v = value.text();
-  dbus_message_iter_append_basic(iter,DBUS_TYPE_STRING,&v);
+
+
+DBusHandlerResult gm_dbus_property_string(DBusConnection * connection,DBusMessage * msg,const FXchar * data) {
+  FXuint serial;
+  DBusMessage * reply = dbus_message_new_method_return(msg);
+  if (reply) {
+    DBusMessageIter iter;
+    dbus_message_iter_init_append(reply,&iter);
+    gm_dbus_variant_append_string(&iter,data);
+    dbus_connection_send(connection,reply,&serial);
+    dbus_message_unref(reply);
+    }
+  return DBUS_HANDLER_RESULT_HANDLED;
+  }
+
+DBusHandlerResult gm_dbus_property_string_list(DBusConnection * connection,DBusMessage * msg,const FXchar * data[]) {
+  FXuint serial;
+  DBusMessage * reply = dbus_message_new_method_return(msg);
+  if (reply) {
+    DBusMessageIter iter;
+    dbus_message_iter_init_append(reply,&iter);
+    gm_dbus_variant_append_string_list(&iter,data);
+    dbus_connection_send(connection,reply,&serial);
+    dbus_message_unref(reply);
+    }
+  return DBUS_HANDLER_RESULT_HANDLED;
   }
 
 
-void gm_dbus_append_string_pair(DBusMessageIter *iter,const FXchar * key,const FXchar * value){
-  dbus_message_iter_append_basic(iter,DBUS_TYPE_STRING,&key);
-  dbus_message_iter_append_basic(iter,DBUS_TYPE_STRING,&value);
+DBusHandlerResult gm_dbus_property_bool(DBusConnection * connection,DBusMessage * msg,const dbus_bool_t data) {
+  FXuint serial;
+  DBusMessage * reply = dbus_message_new_method_return(msg);
+  if (reply) {
+    DBusMessageIter iter;
+    dbus_message_iter_init_append(reply,&iter);
+    gm_dbus_variant_append_bool(&iter,data);
+    dbus_connection_send(connection,reply,&serial);
+    dbus_message_unref(reply);
+    }
+  return DBUS_HANDLER_RESULT_HANDLED;
   }
-  
+
+DBusHandlerResult gm_dbus_property_long(DBusConnection * connection,DBusMessage * msg,const FXlong data) {
+  FXuint serial;
+  DBusMessage * reply = dbus_message_new_method_return(msg);
+  if (reply) {
+    DBusMessageIter iter;
+    dbus_message_iter_init_append(reply,&iter);
+    gm_dbus_variant_append_long(&iter,data);
+    dbus_connection_send(connection,reply,&serial);
+    dbus_message_unref(reply);
+    }
+  return DBUS_HANDLER_RESULT_HANDLED;
+  }
+
+
+DBusHandlerResult gm_dbus_property_double(DBusConnection * connection,DBusMessage * msg,const FXdouble data) {
+  FXuint serial;
+  DBusMessage * reply = dbus_message_new_method_return(msg);
+  if (reply) {
+    DBusMessageIter iter;
+    dbus_message_iter_init_append(reply,&iter);
+    gm_dbus_variant_append_double(&iter,data);
+    dbus_connection_send(connection,reply,&serial);
+    dbus_message_unref(reply);
+    }
+  return DBUS_HANDLER_RESULT_HANDLED;
+  }
+
+
 
 DBusHandlerResult gm_dbus_reply_string(DBusConnection * connection,DBusMessage * msg,const FXchar * data) {
   FXuint serial;
@@ -929,6 +958,6 @@ DBusHandlerResult gm_dbus_reply_if_needed(DBusConnection * connection,DBusMessag
   }
 
 void gm_dbus_match_signal(DBusConnection*connection,const FXchar * path,const FXchar * interface,const FXchar * member){
-  FXString rule = GMStringFormat("type='signal',path='%s',interface='%s',member='%s'",path,interface,member);
-  dbus_bus_add_match(connection,rule.text(),NULL);
+  FXString rule = FXString::value("type='signal',path='%s',interface='%s',member='%s'",path,interface,member);
+  dbus_bus_add_match(connection,rule.text(),nullptr);
   }
