@@ -1,7 +1,7 @@
 /*******************************************************************************
 *                         Goggles Music Manager                                *
 ********************************************************************************
-*           Copyright (C) 2009-2011 by Sander Jansen. All Rights Reserved      *
+*           Copyright (C) 2009-2021 by Sander Jansen. All Rights Reserved      *
 *                               ---                                            *
 * This program is free software: you can redistribute it and/or modify         *
 * it under the terms of the GNU General Public License as published by         *
@@ -18,13 +18,93 @@
 ********************************************************************************/
 #include "gmdefs.h"
 #include "fxext.h"
-#include "GMPlayerManager.h"
-#include "GMPreferences.h"
 
-#if FOXVERSION < FXVERSION(1,7,0)
-#define gm_make_hilite_color makeHiliteColor
-#define gm_make_shadow_color makeShadowColor
-#else
+// Map
+FXDEFMAP(GMThreadDialog) GMThreadDialogMap[]={
+  FXMAPFUNC(SEL_KEYPRESS,0,GMThreadDialog::onKeyPress),
+  FXMAPFUNC(SEL_KEYRELEASE,0,GMThreadDialog::onKeyRelease),
+  FXMAPFUNC(SEL_CLOSE,0,GMThreadDialog::onCmdCancel),
+  FXMAPFUNC(SEL_COMMAND,FXDialogBox::ID_ACCEPT,GMThreadDialog::onCmdAccept),
+  FXMAPFUNC(SEL_CHORE,FXDialogBox::ID_CANCEL,GMThreadDialog::onCmdCancel),
+  FXMAPFUNC(SEL_TIMEOUT,FXDialogBox::ID_CANCEL,GMThreadDialog::onCmdCancel),
+  FXMAPFUNC(SEL_COMMAND,FXDialogBox::ID_CANCEL,GMThreadDialog::onCmdCancel),
+  FXMAPFUNC(SEL_COMMAND,GMThreadDialog::ID_THREAD_EXEC,GMThreadDialog::onThreadExec),
+  };
+
+
+// Object implementation
+FXIMPLEMENT(GMThreadDialog,FXTopWindow,GMThreadDialogMap,ARRAYNUMBER(GMThreadDialogMap))
+
+GMThreadDialog::GMThreadDialog(FXApp* a,const FXString& name,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb,FXint hs,FXint vs):FXTopWindow(a,name,NULL,NULL,opts,x,y,w,h,pl,pr,pt,pb,hs,vs){
+  }
+
+GMThreadDialog::GMThreadDialog(FXWindow* own,const FXString& name,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb,FXint hs,FXint vs):FXTopWindow(own,name,NULL,NULL,opts,x,y,w,h,pl,pr,pt,pb,hs,vs){
+  }
+
+long GMThreadDialog::onThreadExec(FXObject*,FXSelector,void*){
+  create();
+  show();
+  return 1;
+  }
+
+FXuint GMThreadDialog::execute(FXMessageChannel* channel) {
+  mutex.lock();
+  channel->message(this,FXSEL(SEL_COMMAND,ID_THREAD_EXEC),nullptr,0);
+  condition.wait(mutex);
+  mutex.unlock();
+  return code;
+  }
+
+// Close dialog with an accept
+long GMThreadDialog::onCmdAccept(FXObject*,FXSelector,void*){
+  mutex.lock();
+  hide();
+  destroy();
+  code=1;
+  condition.signal();
+  mutex.unlock();
+  return 1;
+  }
+
+// Close dialog with a cancel
+long GMThreadDialog::onCmdCancel(FXObject*,FXSelector,void*){
+  mutex.lock();
+  hide();
+  destroy();
+  code=0;
+  condition.signal();
+  mutex.unlock();
+  return 1;
+  }
+
+// Keyboard press; handle escape to close the dialog
+long GMThreadDialog::onKeyPress(FXObject* sender,FXSelector sel,void* ptr){
+  if(FXTopWindow::onKeyPress(sender,sel,ptr)) return 1;
+  if(((FXEvent*)ptr)->code==KEY_Escape){
+    handle(this,FXSEL(SEL_COMMAND,ID_CANCEL),nullptr);
+    return 1;
+    }
+  return 0;
+  }
+
+// Keyboard release; handle escape to close the dialog
+long GMThreadDialog::onKeyRelease(FXObject* sender,FXSelector sel,void* ptr){
+  if(FXTopWindow::onKeyRelease(sender,sel,ptr)) return 1;
+  if(((FXEvent*)ptr)->code==KEY_Escape){
+    return 1;
+    }
+  return 0;
+  }
+
+
+
+
+
+
+
+
+
+
 
 // Get highlight color
 static FXColor gm_make_hilite_color(FXColor clr){
@@ -56,14 +136,13 @@ static FXColor gm_make_shadow_color(FXColor clr){
   return FXRGB(r,g,b);
   }
 
-#endif
 
 
 
 
 // Fill vertical gradient rectangle
 void fillVerticalGradient(FXDCWindow & dc,FXint x,FXint y,FXint w,FXint h,FXColor top,FXColor bottom){
-  register FXint rr,gg,bb,dr,dg,db,r1,g1,b1,r2,g2,b2,yl,yh,yy,dy,n,t;
+  FXint rr,gg,bb,dr,dg,db,r1,g1,b1,r2,g2,b2,yl,yh,yy,dy,n,t;
   if(0<w && 0<h){
     r1=FXREDVAL(top);
     r2=FXREDVAL(bottom);
@@ -104,7 +183,7 @@ void fillVerticalGradient(FXDCWindow & dc,FXint x,FXint y,FXint w,FXint h,FXColo
 
 // Fill horizontal gradient rectangle
 void fillHorizontalGradient(FXDCWindow & dc,FXint x,FXint y,FXint w,FXint h,FXColor left,FXColor right){
-  register FXint rr,gg,bb,dr,dg,db,r1,g1,b1,r2,g2,b2,xl,xh,xx,dx,n,t;
+  FXint rr,gg,bb,dr,dg,db,r1,g1,b1,r2,g2,b2,xl,xh,xx,dx,n,t;
   if(0<w && 0<h){
     r1=FXREDVAL(left);
     r2=FXREDVAL(right);
@@ -143,7 +222,7 @@ void fillHorizontalGradient(FXDCWindow & dc,FXint x,FXint y,FXint w,FXint h,FXCo
     }
   }
 
-FXIMPLEMENT(GMScrollFrame,FXVerticalFrame,NULL,0)
+FXIMPLEMENT(GMScrollFrame,FXVerticalFrame,nullptr,0)
 
 GMScrollFrame::GMScrollFrame(){
   }
@@ -153,7 +232,7 @@ GMScrollFrame::GMScrollFrame(FXComposite*p):FXVerticalFrame(p,LAYOUT_FILL_X|LAYO
   }
 
 
-FXIMPLEMENT(GMScrollHFrame,FXHorizontalFrame,NULL,0)
+FXIMPLEMENT(GMScrollHFrame,FXHorizontalFrame,nullptr,0)
 
 GMScrollHFrame::GMScrollHFrame(){
   }
@@ -163,7 +242,7 @@ GMScrollHFrame::GMScrollHFrame(FXComposite*p):FXHorizontalFrame(p,LAYOUT_FILL_X|
   }
 
 
-FXIMPLEMENT(GMTabFrame,FXVerticalFrame,NULL,0)
+FXIMPLEMENT(GMTabFrame,FXVerticalFrame,nullptr,0)
 
 GMTabFrame::GMTabFrame(){
   }
@@ -173,7 +252,7 @@ GMTabFrame::GMTabFrame(FXComposite*p):FXVerticalFrame(p,LAYOUT_FILL_X|LAYOUT_FIL
   }
 
 
-FXIMPLEMENT(GMHeaderItem,FXHeaderItem,NULL,0)
+FXIMPLEMENT(GMHeaderItem,FXHeaderItem,nullptr,0)
 
 
 // Map
@@ -199,7 +278,7 @@ GMHeader::GMHeader(FXComposite* p,FXObject* tgt,FXSelector sel,FXuint opts,FXint
 long GMHeader::onPaint(FXObject*,FXSelector,void* ptr){
   FXEvent *ev=(FXEvent*)ptr;
   FXDCWindow dc(this,ev);
-  register FXint x,y,w,h,i,ilo,ihi;
+  FXint x,y,w,h,i,ilo,ihi;
 
   // Set font
   dc.setFont(font);
@@ -265,7 +344,7 @@ long GMHeader::onPaint(FXObject*,FXSelector,void* ptr){
         else if(options&FRAME_RAISED)
           drawRaisedRectangle(dc,0,y,width,h);
         }
-      ((GMHeaderItem*)items[i])->draw(this,dc,0,y,width,h);
+      static_cast<GMHeaderItem*>(items[i])->draw(this,dc,0,y,width,h);
       }
     }
 
@@ -278,10 +357,12 @@ long GMHeader::onPaint(FXObject*,FXSelector,void* ptr){
 
     // Fragment below first item
     if(ilo<0){
+/*
       x=pos;
       if(0<items.no()){
         x=pos+items[0]->getPos();
         }
+*/
 /*
       if(0<x){
         if(options&FRAME_THICK)
@@ -356,7 +437,7 @@ long GMHeader::onPaint(FXObject*,FXSelector,void* ptr){
 
          */
         }
-      ((GMHeaderItem*)items[i])->draw(this,dc,x,0,w,height);
+      static_cast<GMHeaderItem*>(items[i])->draw(this,dc,x,0,w,height);
       }
     }
   return 1;
@@ -364,36 +445,36 @@ long GMHeader::onPaint(FXObject*,FXSelector,void* ptr){
 
 
 
-FXIMPLEMENT(GMMenuCommand,FXMenuCommand,NULL,0)
+FXIMPLEMENT(GMMenuCommand,FXMenuCommand,nullptr,0)
 
 GMMenuCommand::GMMenuCommand(FXComposite* p,const FXString& text,FXIcon* ic,FXObject* tgt,FXSelector sel,FXuint opts) : FXMenuCommand(p,text,ic,tgt,sel,opts){
   backColor=getApp()->getBackColor();
   }
 
 
-FXIMPLEMENT(GMMenuCheck,FXMenuCheck,NULL,0)
+FXIMPLEMENT(GMMenuCheck,FXMenuCheck,nullptr,0)
 
 GMMenuCheck::GMMenuCheck(FXComposite* p,const FXString& text,FXObject* tgt,FXSelector sel,FXuint opts) : FXMenuCheck(p,text,tgt,sel,opts){
   backColor=getApp()->getBackColor();
   }
 
 
-FXIMPLEMENT(GMMenuRadio,FXMenuRadio,NULL,0)
+FXIMPLEMENT(GMMenuRadio,FXMenuRadio,nullptr,0)
 
 GMMenuRadio::GMMenuRadio(FXComposite* p,const FXString& text,FXObject* tgt,FXSelector sel,FXuint opts) : FXMenuRadio(p,text,tgt,sel,opts){
   backColor=getApp()->getBackColor();
   }
 
 
-FXIMPLEMENT(GMMenuCascade,FXMenuCascade,NULL,0)
+FXIMPLEMENT(GMMenuCascade,FXMenuCascade,nullptr,0)
 
 GMMenuCascade::GMMenuCascade(FXComposite* p,const FXString& text,FXIcon* ic,FXPopup* pup,FXuint opts) : FXMenuCascade(p,text,ic,pup,opts){
   backColor=getApp()->getBackColor();
   }
 
-FXIMPLEMENT(GMMenuPane,FXMenuPane,NULL,0)
+FXIMPLEMENT(GMMenuPane,FXMenuPane,nullptr,0)
 
-GMMenuPane::GMMenuPane(FXWindow* owner,FXuint opts) : FXMenuPane(owner,opts) {
+GMMenuPane::GMMenuPane(FXWindow* o,FXuint opts) : FXMenuPane(o,opts) {
   borderColor=getApp()->getShadowColor();
   setFrameStyle(FRAME_LINE);
   }
@@ -411,7 +492,7 @@ GMTextField::GMTextField(FXComposite* p,FXint ncols,FXObject* tgt,FXSelector sel
   }
 
 FXbool GMTextField::extendWordSelection(FXint pos,FXbool /*notify*/) {
-  register FXint sp,ep;
+  FXint sp,ep;
   pos=contents.validate(FXCLAMP(0,pos,contents.length()));
   if(pos<=anchor){
     sp=wordStart(pos);
@@ -467,7 +548,7 @@ long GMTextField::onLeftBtnPress(FXObject*,FXSelector,void* ptr){
   }
 
 
-FXIMPLEMENT(GMSpinner,FXSpinner,NULL,0)
+FXIMPLEMENT(GMSpinner,FXSpinner,nullptr,0)
 
 GMSpinner::GMSpinner(FXComposite* p,FXint ncols,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb) : FXSpinner(p,ncols,tgt,sel,opts,x,y,w,h,pl,pr,pt,pb){
   borderColor=getApp()->getShadowColor();
@@ -940,7 +1021,7 @@ long GMToggleButton::onPaint(FXObject*,FXSelector,void* ptr){
 
 
 
-FXIMPLEMENT(GMRadioButton,FXRadioButton,NULL,0)
+FXIMPLEMENT(GMRadioButton,FXRadioButton,nullptr,0)
 
 
 GMRadioButton::GMRadioButton(){
@@ -989,7 +1070,7 @@ long GMCheckButton::onPaint(FXObject*,FXSelector,void* ptr){
   dc.fillRectangle(ev->rect.x,ev->rect.y,ev->rect.w,ev->rect.h);
 
   // Check background
-  if(check==MAYBE || !isEnabled())
+  if(check==maybe || !isEnabled())
     dc.setForeground(baseColor);
   else
     dc.setForeground(boxColor);
@@ -1006,14 +1087,14 @@ long GMCheckButton::onPaint(FXObject*,FXSelector,void* ptr){
     }
 
   // Check color
-  if(check==MAYBE || !isEnabled())
+  if(check==maybe || !isEnabled())
     dc.setForeground(shadowColor);
   else
     dc.setForeground(checkColor);
 
   // Show as +
   if(options&CHECKBUTTON_PLUS){
-    if(check!=TRUE){
+    if(check!=true){
       dc.fillRectangle(ix+6,iy+4,1,5);
       }
     dc.fillRectangle(ix+4,iy+6,5,1);
@@ -1021,9 +1102,9 @@ long GMCheckButton::onPaint(FXObject*,FXSelector,void* ptr){
 
   // Show as v
   else{
-    if(check!=FALSE){
+    if(check!=false){
       FXSegment seg[6];
-#ifndef WIN32
+#ifndef _WIN32
       seg[0].x1=3+ix; seg[0].y1=5+iy; seg[0].x2=5+ix; seg[0].y2=7+iy;
       seg[1].x1=3+ix; seg[1].y1=6+iy; seg[1].x2=5+ix; seg[1].y2=8+iy;
       seg[2].x1=3+ix; seg[2].y1=7+iy; seg[2].x2=5+ix; seg[2].y2=9+iy;
@@ -1570,10 +1651,10 @@ long GMHeaderButton::onPaint(FXObject*,FXSelector,void*ptr){
   }
 
 
-FXIMPLEMENT(GMScrollArea,FXScrollArea,NULL,0)
+FXIMPLEMENT(GMScrollArea,FXScrollArea,nullptr,0)
 
 void GMScrollArea::replaceScrollbars(FXScrollArea *fs) {
-  GMScrollArea * s = (GMScrollArea*)(fs);
+  GMScrollArea * s = static_cast<GMScrollArea*>(fs);
   delete s->vertical;
   delete s->horizontal;
   delete s->corner;
@@ -1584,10 +1665,10 @@ void GMScrollArea::replaceScrollbars(FXScrollArea *fs) {
 
 
 
-FXIMPLEMENT(GMTreeListBox,FXTreeListBox,NULL,0)
+FXIMPLEMENT(GMTreeListBox,FXTreeListBox,nullptr,0)
 
 void GMTreeListBox::replace(FXTreeListBox *fs) {
-  GMTreeListBox * s = (GMTreeListBox*)(fs);
+  GMTreeListBox * s = static_cast<GMTreeListBox*>(fs);
   GMScrollArea::replaceScrollbars(s->tree);
 
   s->borderColor=s->getApp()->getShadowColor();
@@ -1661,8 +1742,8 @@ void GMScrollBar::drawThumb(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h){
 
 // Handle repaint
 long GMScrollBar::onPaint(FXObject*,FXSelector,void* ptr){
-  register FXEvent *ev=(FXEvent*)ptr;
-  register int total;
+  FXEvent *ev=(FXEvent*)ptr;
+  int total;
   FXDCWindow dc(this,ev);
   if(options&SCROLLBAR_HORIZONTAL){
     total=width-height-height;
@@ -1759,7 +1840,7 @@ long GMScrollCorner::onPaint(FXObject*,FXSelector,void* ptr){
 
 
 
-FXIMPLEMENT(GMTabBook,FXTabBook,NULL,0)
+FXIMPLEMENT(GMTabBook,FXTabBook,nullptr,0)
 
 GMTabBook::GMTabBook(FXComposite* p,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb) :
 	FXTabBook(p,tgt,sel,opts,x,y,w,h,pl,pr,pt,pb) {
@@ -1794,6 +1875,8 @@ GMTabItem::GMTabItem(FXTabBar* p,const FXString& text,FXIcon* ic,FXuint opts,FXi
 //  recalc();
 //  }
 
+
+/// FIXME determine last tab based visibility...
 long GMTabItem::onPaint(FXObject*,FXSelector,void*){
   FXTabBar * bar = (FXTabBar*)getParent();
   FXint tab = bar->indexOfChild(this)/2;
@@ -1801,6 +1884,17 @@ long GMTabItem::onPaint(FXObject*,FXSelector,void*){
   FXint ctab = bar->getCurrent();
 
 
+  FXbool is_last = (tab==(ntabs-1));
+
+  if (!is_last) {
+    is_last=true;
+    for (FXint t=tab+1;t<ntabs;t++){
+      if (bar->childAtIndex(t<<1)->shown()) {
+        is_last=false;
+        break;
+        }
+      }
+    }
 
   FXDCWindow dc(this);
 
@@ -1814,7 +1908,7 @@ long GMTabItem::onPaint(FXObject*,FXSelector,void*){
         dc.fillRectangle(0,0,1,height-1);
 
     /// last one or active one
-    if ((tab == (ntabs-1)) || tab==ctab) {
+    if (is_last || tab==ctab) {
       dc.fillRectangle(width-1,0,1,height-1);
       if (tab!=ctab)
         fillVerticalGradient(dc,1,1,width-2,height-2,gm_make_hilite_color(shadowColor),gm_make_hilite_color(shadowColor));
@@ -1886,7 +1980,7 @@ long GMTabItem::onPaint(FXObject*,FXSelector,void*){
 
 
 
-FXIMPLEMENT(GMListBox,FXListBox,NULL,0);
+FXIMPLEMENT(GMListBox,FXListBox,nullptr,0);
 
 GMListBox::GMListBox(){
   }
@@ -1914,7 +2008,7 @@ void GMListBox::create(){
 
 
 
-FXIMPLEMENT(GMComboBox,FXComboBox,NULL,0);
+FXIMPLEMENT(GMComboBox,FXComboBox,nullptr,0);
 
 GMComboBox::GMComboBox(){
   }
@@ -1939,7 +2033,7 @@ void GMComboBox::create(){
   }
 
 
-FXIMPLEMENT(GMImageFrame,FXImageFrame,NULL,0);
+FXIMPLEMENT(GMImageFrame,FXImageFrame,nullptr,0);
 
 GMImageFrame::GMImageFrame(){
   }
@@ -1952,7 +2046,7 @@ GMImageFrame::GMImageFrame(FXComposite *p,FXImage * img,FXuint opts,FXint x,FXin
 
 
 
-FXIMPLEMENT(GMCoverFrame,FXVerticalFrame,NULL,0);
+FXIMPLEMENT(GMCoverFrame,FXVerticalFrame,nullptr,0);
 
 GMCoverFrame::GMCoverFrame(){
   }
@@ -1964,7 +2058,7 @@ GMCoverFrame::GMCoverFrame(FXComposite *p)
   }
 
 
-FXIMPLEMENT(GMProgressBar,FXProgressBar,NULL,0);
+FXIMPLEMENT(GMProgressBar,FXProgressBar,nullptr,0);
 
 GMProgressBar::GMProgressBar(){
   }
@@ -1977,7 +2071,7 @@ GMProgressBar::GMProgressBar(FXComposite *p,FXObject* tgt,FXSelector sel,FXuint
   setFrameStyle(FRAME_LINE);
   }
 
-#if 0
+
 FXDEFMAP(GMTrackProgressBar) GMTrackProgressBarMap[]={
   FXMAPFUNC(SEL_MOTION,0,GMTrackProgressBar::onMotion),
   FXMAPFUNC(SEL_LEFTBUTTONPRESS,0,GMTrackProgressBar::onLeftBtnPress),
@@ -1997,21 +2091,23 @@ GMTrackProgressBar::GMTrackProgressBar(FXComposite *p,FXObject* tgt,FXSelector s
   barColor=getApp()->getSelbackColor();
   textAltColor=getApp()->getSelforeColor();
   setFrameStyle(FRAME_LINE);
+  barsize=7;
   }
 
 
 
 // Moving
-long GMTrackProgressBar::onMotion(FXObject*,FXSelector,void* ptr){
-  register FXEvent *event=(FXEvent*)ptr;
-  register FXint xx,yy,ww,hh,lo,hi,p,h,travel;
+long GMTrackProgressBar::onMotion(FXObject*,FXSelector,void*){
+//  register FXEvent *event=(FXEvent*)ptr;
+  FXuint /*xx,yy,ww,hh,lo,hi,*/p=0/*,h,travel*/;
   if(!isEnabled()) return 0;
   if(flags&FLAG_PRESSED){
+/*
+
     yy=border+padtop;
     xx=border+padleft;
     hh=height-(border<<1)-padtop-padbottom;
     ww=width-(border<<1)-padleft-padright;
-/*
     if(options&PROGRESSBAR_VERTICAL){
       h=event->win_y-dragpoint;
       travel=hh-headsize;
@@ -2062,9 +2158,8 @@ long GMTrackProgressBar::onMotion(FXObject*,FXSelector,void* ptr){
 
 // Pressed LEFT button
 long GMTrackProgressBar::onLeftBtnPress(FXObject*,FXSelector,void* ptr){
-  fxmessage("press\n");
-  register FXEvent *event=(FXEvent*)ptr;
-  register FXint p=progress;
+//  register FXEvent *event=(FXEvent*)ptr;
+//  register FXint p=progress;
   flags&=~FLAG_TIP;
   handle(this,FXSEL(SEL_FOCUS_SELF,0),ptr);
   if(isEnabled()){
@@ -2085,7 +2180,6 @@ long GMTrackProgressBar::onLeftBtnPress(FXObject*,FXSelector,void* ptr){
         }
       }
     else{
-
       if(event->win_x<headpos){
         p=progress-incr;
         }
@@ -2099,13 +2193,16 @@ long GMTrackProgressBar::onLeftBtnPress(FXObject*,FXSelector,void* ptr){
 
       }
 */
+
 //    if(p<range[0]) p=range[0];
 //    if(p>range[1]) p=range[1];
+/*
     if(p!=progress){
       setProgress(p);
       flags|=FLAG_CHANGED;
       if(target) target->tryHandle(this,FXSEL(SEL_CHANGED,message),(void*)(FXival)progress);
       }
+*/
     return 1;
     }
   return 0;
@@ -2114,9 +2211,8 @@ long GMTrackProgressBar::onLeftBtnPress(FXObject*,FXSelector,void* ptr){
 
 // Released Left button
 long GMTrackProgressBar::onLeftBtnRelease(FXObject*,FXSelector,void* ptr){
-  register FXEvent *event=(FXEvent*)ptr;
-  fxmessage("release\n");
-  register FXuint flgs=flags;
+  FXEvent *event=(FXEvent*)ptr;
+//  register FXuint flgs=flags;
   if(isEnabled()){
     ungrab();
 
@@ -2139,127 +2235,3 @@ long GMTrackProgressBar::onLeftBtnRelease(FXObject*,FXSelector,void* ptr){
   return 0;
   }
 
-#endif
-
-
-
-
-
-
-
-
-
-
-
-
-
-FXIMPLEMENT(GMSettings,FXSettings,NULL,0);
-
-#if FOXVERSION < FXVERSION(1,7,0)
-// Parse filename
-FXbool GMSettings::parseFile(const FXString& filename,FXbool mrk){
-  FXFile file(filename,FXIO::Reading);
-  if(file.isOpen()){
-
-    // Prepare buffer string
-    FXString string('\0',file.size());
-
-    // Load file
-    if(file.readBlock((void*)string.text(),string.length())==string.length()){
-      FXStringDict *group=NULL;
-      FXint lineno=1,p=0,b,e;
-      FXString name;
-      FXString value;
-
-      // Skip BOM, if any
-      if(string[p]=='\xef' && string[p+1]=='\xbb' && string[p+2]=='\xbf') p+=3;
-
-      // Parse one line at a time
-      while(string[p]){
-
-        // Skip leading blanks
-        while(Ascii::isBlank(string[p])) p++;
-
-        // Non-comment
-        if(string[p] && string[p]!='\n' && string[p]!='\r' && string[p]!='#' && string[p]!=';'){
-
-          // Parse section name
-          if(string[p]=='['){
-
-            b=++p;
-
-            // Scan section name
-            while(string[p] && string[p]!=']' && string[p]!='\n' && string[p]!='\r' && !Ascii::isControl(string[p])) p++;
-
-            // Check errors
-            if(string[p]!=']'){ fxwarning("%s:%d: illegal section name.\n",filename.text(),lineno); goto next; }
-
-            e=p++;
-
-            // Grab name
-            name=string.mid(b,e-b);
-
-            // Add new section dict
-            group=insert(name.text());
-            }
-
-          // Parse name-value pair
-          else{
-
-            // Should have seen section prior to this
-            if(!group){ fxwarning("%s:%d: settings entry should follow a section.\n",filename.text(),lineno); goto next; }
-
-            b=p;
-
-            // Scan key name
-            while(string[p] && string[p]!='=' && string[p]!='\n' && string[p]!='\r' && !Ascii::isControl(string[p])) p++;
-
-            // Check errors
-            if(string[p]!='='){ fxwarning("%s:%d: expected '=' to follow key.\n",filename.text(),lineno); goto next; }
-
-            e=p++;
-
-            // Remove trailing spaces after name
-            while(b<e && Ascii::isBlank(string[e-1])) e--;
-
-            // Grab name
-            name=string.mid(b,e-b);
-
-            // Skip leading spaces
-            while(Ascii::isBlank(string[p])) p++;
-
-            // Mark value
-            b=p;
-
-            // Scan value
-            while(string[p] && string[p]!='\n' && string[p]!='\r' && !Ascii::isControl(string[p])) p++;
-
-            e=p;
-
-            // Remove trailing spaces after value
-            while(b<e && Ascii::isBlank(string[e-1])) e--;
-
-            // Grab the unescaped value
-            value=dequote((FXchar*)string.mid(b,e-b).text());
-
-            // Add entry to current section
-            group->replace(name.text(),value.text(),mrk);
-            }
-          }
-
-        // Skip to end of line
-next:   while(string[p] && string[p]!='\n') p++;
-
-        // End of line
-        if(string[p]=='\n'){
-          lineno++;
-          p++;
-          }
-        }
-      return true;
-      }
-    }
-  return false;
-  }
-#endif
-
